/*

RenAttach 0.13 for UNIX / Linux
Copyright(C) 2000, Jem E. Berkes

Posted at www.pc-tools.net on 2000-12-10
Developer: Jem Berkes <berkes@pc9.org>

This software is provided "as is" and without any warranties as to
performance, merchantability, fitness for a particular purpose, or
any other warranties whether expressed or implied. The entire risk
as to the results and performance of the software is assumed by you.
The author shall not have any liability to you or any other person
or entity for any damages whatsoever, including, but not limited to,
loss of revenue or profit, lost or damaged data or other. The author
is also not responsible for claims by a third party.

You may distribute this software in its original, unmodified form only.
You are specifically prohibited from charging, or requesting donations,
for any such copies, however made; and from distributing the software
and/or documentation with other products (commercial or otherwise)
without prior written permission from the author.

By compiling and using the software, you are agreeing with these terms
and conditions.

*/

#define MAILER_CMD "/usr/bin/procmail"
#define EXTENSIONS "COM,EXE,BAT,CMD,VBS,VBE,JS,JSE,WSF,WSH,REG,SCR,PIF"

#include <stdio.h>
#include <string.h>
#include <unistd.h>


const char VERSION[]="RenAttach 0.13 from www.pc-tools.net";
const int MAX_LINE = 512;
char scratch[MAX_LINE];
FILE *mailer;
bool rename_all = true;


int syntax(char *exec)
{
fprintf(stderr, "\n%s\nDeveloped by Jem Berkes\n\n", VERSION);
fprintf(stderr, "Usage: %s [-f | -p]\n", exec);
fprintf(stderr, "-full will rename all attachments\n");
fprintf(stderr, "-partial will rename only the following extensions:\n");
fprintf(stderr, "\t%s\n\n", EXTENSIONS);
fprintf(stderr, "To filter incoming mail, enter one of the following in .forward\n");
fprintf(stderr, "\"|/path/renattach -f\"\n");
fprintf(stderr, "\"|/path/renattach -p\"\n\n");
return -1;
}


bool is_match(char *filename)
{
char extcopy[MAX_LINE];
char *ext;

ext = strchr(filename, '.');
if (ext == NULL)
	return false;
else	{
	ext++;
	char *token;
	strcpy(extcopy, EXTENSIONS);
	token = strtok(extcopy, ",");
	while (token != 0)
		{
		if (strcasecmp(token, ext) == 0)
			return true;
		token = strtok(NULL, ",");
		}
	return false;
	}
}


bool make_safe(char *orig_line, char *name)
{
char filename[MAX_LINE];

if (name[0]=='\n' || name[0]==0)
	return false;
if ((sscanf(name, "%[^\"]", filename)==1) ||
	(sscanf(name, "\"%[^\"]", filename)==1) ||
	(sscanf(name, "%[^\n]", filename)==1))
	{
	if (filename[strlen(filename)-1] == '\n')
		filename[strlen(filename)-1] = 0;
	}
else
	return false;
int n = (strstr(orig_line, filename)-orig_line);
strncpy(scratch, orig_line, n);
scratch[n] = 0;
char *txtend = strstr(orig_line, filename) + strlen(filename);
if (rename_all)
	{
	if (filename[strlen(filename)-1] == '.')
		strcat(filename, "rename");
	else
		strcat(filename, ".rename");
	}
else
	if (is_match(filename))
		strcat(filename, ".rename");
strcat(scratch, filename);
strcat(scratch, txtend);
fputs(scratch, mailer);
return true;
}


int main (int argc, char *argv[])
{
char line[MAX_LINE];
char *fname;
bool inheader = true;
bool mime_type = false;

if ((argc==1) || (argv[1][0] != '-'))
	return syntax(argv[0]);
switch (argv[1][1])
	{
	case 'f':
	case 'F':
		rename_all = true;
		break;

	case 'p':
	case 'P':
		rename_all = false;
		break;

	default:
		return syntax(argv[0]);
	}
mailer = popen(MAILER_CMD, "w");
if (mailer == NULL) return -1;
while (fgets(line, MAX_LINE, stdin))
	{
        if (inheader && line[0]=='\n')
                {
                inheader = false;
		if (rename_all) strcpy(scratch, "full");
		else strcpy(scratch, "partial");
                fprintf(mailer, "X-Filtered-With: %s - %s filter\n\n", VERSION, scratch);
		continue;
                }
        if (!mime_type && (strncasecmp(line, "Content-type:", 13)==0))
                mime_type = true;
        if (mime_type && ( (fname=strstr(line, "name="))!=NULL ||
                                (fname=strstr(line, "NAME="))!=NULL ) )
                {
                if (make_safe(line, fname+5))
                        continue;
                }
	else if (sscanf(line, "begin %*d %[^\n]", scratch)==1)
		{
		if (make_safe(line, strstr(line, scratch)))
			continue;
		}
	else if (mime_type && line[0]=='\n')
		mime_type = false;
	fputs(line, mailer);
	}
}
